#ifndef SECURE_CHANNEL_T_H__
#define SECURE_CHANNEL_T_H__

#include <stdint.h>
#include <wchar.h>
#include <stddef.h>
#include <stdio.h>
#include "sgx_edger8r.h" /* for sgx_ocall etc. */

#include "stdbool.h"
#include "secgear_urts.h"

#include <stdlib.h> /* for size_t */

#define SGX_CAST(type, item) ((type)(item))

#ifdef __cplusplus
extern "C" {
#endif

#ifndef PRINT_LEVEL
#define PRINT_LEVEL 0
#endif
#define PRINT_ERROR 0
#define PRINT_WARNING 1
#define PRINT_STRACE 2
#define PRINT_DEBUG 3
int get_enclave_pubkey(size_t* session_id, uint8_t* pubkey, size_t* pubkey_len);
int init_session(size_t* session_id);
int set_enc_key(size_t session_id, uint8_t* data, size_t data_len);
int get_enclave_exch_param_len(size_t session_id, size_t* exch_param_len);
int get_enclave_exch_param(size_t session_id, uint8_t* exch_param, size_t exch_param_len);
int set_peer_exch_param(size_t session_id, uint8_t* data, size_t data_len);
void del_enclave_sec_chl(size_t session_id);
int enclave_start_sec_chl(void);
void enclave_stop_sec_chl(void);
void enclave_check_session_timeout(void);

sgx_status_t SGX_CDECL ocall_cc_enclave_PrintInfo(const char* str);
sgx_status_t SGX_CDECL sgx_oc_cpuidex(int cpuinfo[4], int leaf, int subleaf);
sgx_status_t SGX_CDECL sgx_thread_wait_untrusted_event_ocall(int* retval, const void* self);
sgx_status_t SGX_CDECL sgx_thread_set_untrusted_event_ocall(int* retval, const void* waiter);
sgx_status_t SGX_CDECL sgx_thread_setwait_untrusted_events_ocall(int* retval, const void* waiter, const void* self);
sgx_status_t SGX_CDECL sgx_thread_set_multiple_untrusted_events_ocall(int* retval, const void** waiters, size_t total);
sgx_status_t SGX_CDECL u_sgxssl_ftime(void* timeptr, uint32_t timeb_len);
sgx_status_t SGX_CDECL ocall_cc_read(int* retval, int fd, void* buf, size_t buf_len);
sgx_status_t SGX_CDECL ocall_cc_write(int* retval, int fd, const void* buf, size_t buf_len);
sgx_status_t SGX_CDECL ocall_cc_getenv(int* retval, const char* name, size_t name_len, void* buf, int buf_len, int* need_len);
sgx_status_t SGX_CDECL ocall_cc_fopen(uint64_t* retval, const char* filename, size_t filename_len, const char* mode, size_t mode_len);
sgx_status_t SGX_CDECL ocall_cc_fclose(int* retval, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_ferror(int* retval, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_feof(int* retval, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_fflush(int* retval, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_ftell(long int* retval, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_fseek(int* retval, uint64_t fp, long int offset, int origin);
sgx_status_t SGX_CDECL ocall_cc_fread(size_t* retval, void* buf, size_t total_size, size_t element_size, size_t cnt, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_fwrite(size_t* retval, const void* buf, size_t total_size, size_t element_size, size_t cnt, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_fgets(int* retval, char* str, int max_cnt, uint64_t fp);
sgx_status_t SGX_CDECL ocall_cc_fputs(int* retval, const char* str, size_t total_size, uint64_t fp);
sgx_status_t SGX_CDECL pthread_wait_timeout_ocall(int* retval, unsigned long long waiter, unsigned long long timeout);
sgx_status_t SGX_CDECL pthread_create_ocall(int* retval, unsigned long long self);
sgx_status_t SGX_CDECL pthread_wakeup_ocall(int* retval, unsigned long long waiter);
void PrintInfo(int level, const char *fmt, ...);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
